package com.hw.controller;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.github.pagehelper.PageInfo;
import com.hw.form.WechatInfoForm;
import com.hw.service.WechatInfoService;
import com.hw.util.EasyHttpUtils;
import com.hw.util.RedisUtils;
import com.hw.util.StringUtils;
import com.hw.util.base.BaseController;
import com.hw.util.base.ResponseBase;
import com.hw.util.base.ResultVO;
import com.hw.util.exception.GlobalException;
import io.swagger.annotations.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.*;
import java.util.concurrent.TimeUnit;

/**
 * 原公众号openid与新公众号openid转换
 * lym
 */
@RestController
@RequestMapping("/wechatOpenid")
@Api(description="原公众号openid与新公众号openid转换")
public class WechatOpenidController extends BaseController {

    private Logger log = LoggerFactory.getLogger(WechatOpenidController.class);

    @Autowired
    private WechatInfoService wechatInfoService;

    @Autowired
    private RedisUtils redisUtils;

    /**
     * 设置微信的appid
     */
    @Value("${wx.ma.appId}")
    private String oldAppId;

    /**
     * 设置微信的app secret
     */
    @Value("${wx.ma.secret}")
    private String oldSecret;

    /**
     * 新微信的appid
     */
    @Value("${newwx.ma.appId}")
    private String newAppId;

    /**
     * 新微信的app secret
     */
    @Value("${newwx.ma.secret}")
    private String newSecret;

    private final static String CARBONOLDWHAT_ACCESS_TOKEN = "carbonoldwhat_access_token";

    private final static String CARBONNEWWHAT_ACCESS_TOKEN = "carbonnewwhat_access_token";

    private final static String WECHAT_API_URl = "https://api.weixin.qq.com/cgi-bin/";

    /**
     * 获取旧公众号openid
     * lym
     * @return
     */
    @ApiOperation(value = "获取原公众号关注用户openid",notes = "获取原公众号关注用户openid")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "nextOpenid",value = "第一个openid",dataType = "String")
    })
    @ApiResponse(code = 0,message = "操作成功")
    @GetMapping("/getOpenids")
    public ResponseBase getOpenids(String nextOpenid){
        if (StringUtils.isEmpty(nextOpenid)){
            nextOpenid = "";
        }
        int addNum = addOpenid(nextOpenid,0,0);
        ResultVO resultVO = new ResultVO();
        if (addNum == 0){
            resultVO.setMsg("没有可拉取的openid");
        }else {
            resultVO.setMsg("拉取保存openid"+ addNum +"条");
        }
        return responseSuccess(resultVO);
    }

    /**
     *  获取access_token，一天2000次，一次获取的token两小时内有效
     *  lym
     * @return
     */
    private String getAccessToken(String wxAppid,String wxSecret,String accessTokenRedisKey){
        if (redisUtils.existsKey(accessTokenRedisKey)){
            return (String) redisUtils.getValue(accessTokenRedisKey);
        }
        String get = EasyHttpUtils.LoadInstance().get(WECHAT_API_URl +"/token?grant_type=client_credential&appid="+ wxAppid +"&secret=" + wxSecret);
        log.info(get);
        if (StringUtils.isNotEmpty(get)) {
            JSONObject jsonObject = JSONObject.parseObject(get);
            if (jsonObject.containsKey("access_token")){
                //缓存
                int time = (int) jsonObject.get("expires_in");
                redisUtils.setValue(accessTokenRedisKey,jsonObject.getString("access_token"),time, TimeUnit.SECONDS);
                return jsonObject.getString("access_token");
            }
        }

        log.info("获取token失败");
        throw new GlobalException("获取token失败,请联系管理员!");
    }

    /**
     * 一次拉取调用最多拉取10000个关注者的OpenID，可以通过多次拉取的方式来满足,上一次调用得到的返回中的next_openid值，作为下一次调用中的next_openid值
     * lym
     * @param nextOpenid
     */
    private int addOpenid(String nextOpenid,int num,int addNum) {
        //获取access_token
        String accessToken = getAccessToken(oldAppId,oldSecret,CARBONOLDWHAT_ACCESS_TOKEN);
        //获取关注用户
        String getChatopenids = EasyHttpUtils.LoadInstance().get(WECHAT_API_URl +"/user/get?access_token=" + accessToken + "&next_openid=" + nextOpenid);
        log.info(getChatopenids);
        if (getChatopenids.contains("errcode")){
            throw new GlobalException("获取openids失败,请联系管理员!");
        }
        if (StringUtils.isNotEmpty(getChatopenids) && getChatopenids.contains("data")
                && getChatopenids.contains("openid") && !getChatopenids.contains("errcode")) {
            JSONObject jsonObject = JSONObject.parseObject(getChatopenids);
            int gzTotal = 0, getTotal = 0;
            List<String> openids = new ArrayList();
            if (jsonObject.containsKey("total")) {//关注用户数
                gzTotal = jsonObject.getInteger("total");
            }
            if (jsonObject.containsKey("count")) {//获取用户数
                getTotal = jsonObject.getInteger("count");
                num += getTotal;
            }
            if (jsonObject.containsKey("data")) {
                String data = jsonObject.getString("data");
                if (StringUtils.isNotEmpty(data) && data.contains("openid")) {
                    JSONObject dataObject = JSONObject.parseObject(data);
                    JSONArray openidArr = dataObject.getJSONArray("openid");
                    //openids = Arrays.asList(dataObject.getString("openid").replace(",",""));
                    for (int i = 0;i < openidArr.size();i++){
                        openids.add(openidArr.getString(i));
                    }
                    //保存关注用户openid
                    int i = wechatInfoService.batchAdd(openids,oldAppId);
                    log.info("保存关注用户openid:"+ i);
                    if (i > 0){
                        addNum += i;
                    }

                }
            }

            if (num < gzTotal){ //多次拉取
                if (jsonObject.containsKey("next_openid")){
                    nextOpenid = jsonObject.getString("next_openid");
                }
                addOpenid(nextOpenid,num,addNum);
            }
        }

        return addNum;
    }

    /**
     * 转换openid
     * lym
     * @return
     */
    @ApiOperation(value = "迁移后新旧openid转换",notes = "迁移后新旧openid转换")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "appid",value = "旧公众号appid",dataType = "String"),
            @ApiImplicitParam(name = "pageSize",value = "每次转换数量，最多100",dataType = "int")
    })
    @ApiResponse(code = 0,message = "操作成功")
    @GetMapping("/changeOpenids")
    public ResponseBase changeOpenids(String appid,@RequestParam(defaultValue = "100",required = false) int pageSize){
        if (StringUtils.isEmpty(appid)){
            appid = oldAppId;
        }
        if (pageSize > 100){
            pageSize = 100;
        }

        ResultVO resultVO = new ResultVO();
        int editNum = change(appid,1,pageSize,0);
        if (editNum < 1){
            resultVO.setMsg("无可转换openid,请确认原公众号与目标公众号是否存在迁移关系!");
        }else {
            resultVO.setMsg("转换完成");
        }
        return responseSuccess(resultVO);
    }

    /**
     * 转换
     * lym
     * @param appid
     * @param editNum
     * @return
     */
    private int change(String appid,int pageNo,int pageSize,int editNum){
        WechatInfoForm from = new WechatInfoForm();
        from.setOldAppid(appid);
        from.setPageNo(pageNo);
        from.setPageSize(pageSize); //因一次只能转换100个，所以取100个
        PageInfo<List<String>> listPageInfo = wechatInfoService.selectOldOpenids(from);
        int total = (int) listPageInfo.getTotal();
        if (total > 0) {
            List<List<String>> openidList = listPageInfo.getList();
            Map param = new HashMap();
            param.put("from_appid", oldAppId); //原帐号的appid
            param.put("openid_list", openidList); //需要转换的openid
            //获取新公众号access_token
            String accessToken = getAccessToken(newAppId, newSecret, CARBONNEWWHAT_ACCESS_TOKEN);
            //帐号迁移15天后，该转换接口将会失效、无法拉取到数据
            String result = EasyHttpUtils.LoadInstance().post(WECHAT_API_URl +"/changeopenid?access_token=" + accessToken, (JSONObject) JSONObject.toJSON(param));
            log.info(result);
            if (StringUtils.isNotEmpty(result)){
                JSONObject jsonObject = JSONObject.parseObject(result);
                if (jsonObject.containsKey("errcode") && jsonObject.getInteger("errcode") == 0 && jsonObject.containsKey("result_list")){
                    JSONArray resultList = (JSONArray) jsonObject.get("result_list");
                    if (resultList != null && resultList.size() > 0){ //组装对应openid
                        List<WechatInfoForm> editList = new ArrayList<>();
                        WechatInfoForm infoFrom = null;
                        for (int i = 0;i < resultList.size();i++){
                            JSONObject object = resultList.getJSONObject(i);
                            if (object.containsKey("err_msg") && "ok".equals(object.getString("err_msg"))){
                                infoFrom = new WechatInfoForm();
                                infoFrom.setOldOpenid(object.getString("ori_openid"));
                                infoFrom.setNewOpenid(object.getString("new_openid"));
                                infoFrom.setOldAppid(appid);
                                editList.add(infoFrom);
                            }
                        }

                        if (editList != null && editList.size() > 0){
                           int i = wechatInfoService.batchUpdate(editList); //修改迁移对应openid
                            if (i > 0){
                                editNum += i;
                            }
                        }

                        int pageNum = total/pageSize;
                        int yNum = total%pageSize;
                        if (yNum > 0){ //不整除分页加1
                            pageNum += 1;
                        }
                        if (pageNo < pageNum){
                            pageNo += 1;
                            change(appid,pageNo,pageSize,editNum);
                        }
                    }
                }
                if (jsonObject.containsKey("errcode") && jsonObject.getInteger("errcode") != 0){
                    throw new GlobalException("openid转换失败,请检查参数或确认原公众号与目标公众号是否存在迁移关系!!");
                }
            }

            /* 测试批量修改 */
            /*List<WechatInfoForm> editList = new ArrayList<>();
            WechatInfoForm infoFrom = null;
            for (int i = 0; i < openidList.size(); i++) {
                infoFrom = new WechatInfoForm();
                infoFrom.setOldOpenid(String.valueOf(openidList.get(i)));
                infoFrom.setNewOpenid(String.valueOf(new Random().nextInt(20)));
                infoFrom.setOldAppid(appid);
                editList.add(infoFrom);
            }


            if (editList != null && editList.size() > 0) {
                int i = wechatInfoService.batchUpdate(editList); //修改迁移对应openid
                if (i > 0) {
                    editNum += i;
                }
            }*/

            int pageNum = total/pageSize;
            int yNum = total%pageSize;
            if (yNum > 0){ //不整除分页加1
                pageNum += 1;
            }
            if (pageNo < pageNum){
                pageNo += 1;
                change(appid,pageNo,pageSize,editNum);
            }
        }
        return editNum;
    }
}
