package com.hw.util.exception;


import com.hw.util.base.ResponseBase;
import com.hw.util.base.ResultVO;
import com.hw.util.base.RetHead;
import com.hw.util.enums.ResultEnum;
import lombok.extern.slf4j.Slf4j;
import org.springframework.dao.CannotAcquireLockException;
import org.springframework.http.converter.HttpMessageNotReadableException;
import org.springframework.util.CollectionUtils;
import org.springframework.validation.ObjectError;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.bind.MissingServletRequestParameterException;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.RestControllerAdvice;

import javax.servlet.http.HttpServletRequest;
import java.io.FileNotFoundException;
import java.util.List;


@Slf4j
@CrossOrigin
@RestControllerAdvice
public class GlobalDefaultExceptionHandler {

    @ExceptionHandler(value = CommitException.class)
    public ResponseBase defaultErrorHandler(HttpServletRequest req, CommitException e) {
        log.error(req.getRequestURL() + "<:=====:>" +
                req.getMethod() + "<:=====:>" +
                req.getQueryString() + "<:=====:>" +
                e.getMessage());
        e.printStackTrace();
        RetHead retHead = new RetHead();
        retHead.setErrCode((short) 1);
        retHead.setErrMsg(e.getMessage());
        return new ResponseBase(retHead);
    }
    @ExceptionHandler(value = CannotAcquireLockException.class)
    public ResponseBase defaultErrorHandler(HttpServletRequest req, CannotAcquireLockException e) {
        log.error(req.getRequestURL() + "<:=====:>" +
                req.getMethod() + "<:=====:>" +
                req.getQueryString() + "<:=====:>" +
                e.getMessage());
        e.printStackTrace();
        RetHead retHead = new RetHead();
        retHead.setErrCode((short) 1);
        retHead.setErrMsg("当前系统整理数据稍后再试!");
        return new ResponseBase(retHead);
    }


    @ExceptionHandler(value = Exception.class)
    public ResponseBase defaultErrorHandler(
            HttpServletRequest req, Exception e) {

        log.error(req.getRequestURL() + "<:=====:>" +
                req.getMethod() + "<:=====:>" +
                req.getQueryString() + "<:=====:>" +
                e.getMessage());
        e.printStackTrace();

        ResultVO resultVO = new ResultVO();

        if (e instanceof RuntimeException) {
            log.error(e.getClass().toString());
            resultVO.setCode(1);
            resultVO.setMsg(e.getMessage());
            //resultVO.setMsg("操作失败！");
        } else if (e instanceof GlobalException) {
            log.error(e.getClass().toString());
            GlobalException globalException = (GlobalException) e;
            Integer code = globalException.getCode();
            String msg = globalException.getMessage();
            resultVO.setCode(code);
            resultVO.setMsg(msg);
        } else if (e instanceof MethodArgumentNotValidException) {
            log.error(e.getClass().toString());
            List<ObjectError> allErrors = ((MethodArgumentNotValidException) e).getBindingResult()
                    .getAllErrors();
            if (!CollectionUtils.isEmpty(allErrors)) {
                String errorMessage = allErrors.get(0).getDefaultMessage();
                log.error(errorMessage);
                resultVO.setCode(ResultEnum.FAILURE.getCode());
                resultVO.setMsg(errorMessage);
            }
        } else if (e instanceof MissingServletRequestParameterException) {
            log.error(e.getClass().toString());
            resultVO.setCode(ResultEnum.PARAMETER_NOT_NULL.getCode());
            resultVO.setMsg(ResultEnum.PARAMETER_NOT_NULL.getMsg());
        } else if (e instanceof HttpMessageNotReadableException) {
            log.error(e.getClass().toString());
            resultVO.setCode(ResultEnum.PARAMETER_NOT_NULL.getCode());
            resultVO.setMsg(ResultEnum.PARAMETER_NOT_NULL.getMsg());
        } else if (e instanceof FileNotFoundException) {
            log.error(e.getClass().toString());
            resultVO.setCode(ResultEnum.FILE_NOT_FOUND.getCode());
            resultVO.setMsg(ResultEnum.FILE_NOT_FOUND.getMsg());
        } else {
            log.error(e.getClass().toString() + ":" + e.getMessage());
            resultVO.setCode(ResultEnum.FAILURE.getCode());
            resultVO.setMsg(ResultEnum.FAILURE.getMsg());
        }

        RetHead retHead = new RetHead();
        retHead.setErrCode(resultVO.getCode().shortValue());
        retHead.setErrMsg(resultVO.getMsg());
        return new ResponseBase(retHead);
    }
}
